/**
 * Copyright (C) 2005-2011 by Rivello Multimedia Consulting (RMC).                    
 * info@RivelloMultimediaConsulting.com                                                  
 *                                                                      
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the      
 * "Software"), to deal in the Software without restriction, including  
 * without limitation the rights to use, copy, modify, merge, publish,  
 * distribute, sublicense, and#or sell copies of the Software, and to   
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:                                            
 *                                                                      
 * The above copyright notice and this permission notice shall be       
 * included in all copies or substantial portions of the Software.      
 *                                                                      
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,      
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF   
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR    
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.                                      
 */
// Marks the right margin of code *******************************************************************
package
{


	// --------------------------------------
	// Imports
	// --------------------------------------
	import flash.display.Sprite;
	import com.rmc.demos.frametagmetadatademo.managers.FrameTagMetadataDemoAssetManager;
	import flash.events.Event;


	// --------------------------------------
	// Metadata
	// --------------------------------------
	/**
	 * Using the Frame Tag is optional. It creates an automatic, non-configurable preloader within your 1 existing swf.
	 * 
	 * Include ONLY for NON-Flex projects
	 * Include ONLY when no other preloader is implemented.
	 * Include ONLY as shown. No modifications, except change extraClass to be the name of the current class
	 * 
	 * NOTE:  When in use, the parent of the display list will be a SystemManager instance, 
	 *        not your main application. Typically this difference affects nothing.
	 * 
	 * NOTE:  When used, you'll see a harmless 'Default CSS file not found'. 
	 *        To solve the warning you can do this;
	 *        
	 *        a) Add an empty MainStyles.css class adjacent to this class. Advanced users can locate it elsewhere.
	 *        b) Add this compiler argument if desired to remove the 
	 *           warning [defaults-css-url "MainStyles.css"] without the []
	 * 
	 */
	[Frame(factoryClass="com.rmc.demos.frametagmetadatademo.managers.SystemManager")]
	
	// --------------------------------------
	// Class
	// --------------------------------------
	/**
	 * <p>The <code>FrameTagMetadataDemo</code> class is designed to show how to delay main document class creation by 1 frame and instead
	 * front-load another manager class (for custom 1-swf preloading scenarios and more).</p>
	 * 
	 * <p>NOTE: This class 'knows' nothing about itself being preloader. Except it has a Frame metadata tag.  Nice. </p>
	 * 
	 * <p>AUTHOR  : Samuel Asher Rivello</p>
	 * <p>COMPANY : RivelloMultimediaConsulting.com</p>
	 * <p>CREATION DATE    : Apr 05, 2011</p>
	 * 
	 * @example Here is a code example. 
	 *  
	 * <listing version="3.0">
	 * <p>Describe this code example.</p>
	 *
	 * </listing>
	 *
	 */
	public class FrameTagMetadataDemo extends Sprite
	{

		// --------------------------------------
		// Properties
		// --------------------------------------
		// PUBLIC GETTER/SETTERS

		// PUBLIC CONST

		// PRIVATE


		// --------------------------------------
		// Constructor
		// --------------------------------------
		/**
		 * This is the constructor.
		 * 
		 */
		public function FrameTagMetadataDemo()
		{
			// SUPER
			super();

			// EVENTS
			addEventListener(Event.ADDED_TO_STAGE, _onAddedToStage);

			// VARIABLES

			// PROPERTIES

			// METHODS

		}


		// --------------------------------------
		// Methods
		// --------------------------------------
		// PUBLIC

		// PRIVATE
		/**
		 * Layout 
		 * 
		 * @return void
		 */
		private function _doLayout() : void
		{
			//ADD A LARGE-K IMAGE. NOTE THIS K IS NOT LOADED UNTIL *AFTER* THE PRELOADER APPEARS. NICE.
			var staircaseInIndiaSprite : Sprite 	= new FrameTagMetadataDemoAssetManager.StaircaseInIndiaSprite();
			var originalHeight_num	: Number		= staircaseInIndiaSprite.height;
			staircaseInIndiaSprite.height 			= stage.stageHeight;
			staircaseInIndiaSprite.scaleX			= (staircaseInIndiaSprite.height/originalHeight_num); //make the image fullscreen, but at original proportions
			staircaseInIndiaSprite.x 				= stage.stageWidth / 2 - staircaseInIndiaSprite.width / 2;
			staircaseInIndiaSprite.y 				= stage.stageHeight / 2 - staircaseInIndiaSprite.height / 2;
			addChild(staircaseInIndiaSprite);

		}

		// --------------------------------------
		// Event Handlers
		// --------------------------------------
		/**
		 * Handles the event: <code>Event.ADDED_TO_STAGE</code>.
		 * 
		 * @param event <code>Event</code> The incoming event payload.
		 *  
		 * @return void
		 * 
		 */
		private function _onAddedToStage(event : Event) : void
		{
			_doLayout();
			
			//NOTE, THAT BY DEFAULT, THE MAIN CLASSES PARENT IS NOT ROOT ANYMORE, ITS THE SYSTEM MANAGER
			//IF THIS IS A PROBLEM, YOU CAN HAVE THE SYSTEMPARENT REMOVE ITSELF AND ATTACH THIS CLASS
			//TO *ITS PARENT*. Then this classes' parent is root. Like a normal as3-application.
			trace ("This.parent : " + parent);
		}
	}
}
    
