/**
 * Copyright (C) 2005-2011 by Rivello Multimedia Consulting (RMC)                     
 * info@RivelloMultimediaConsulting.com                                                  
 *                                                                      
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the      
 * "Software"), to deal in the Software without restriction, including  
 * without limitation the rights to use, copy, modify, merge, publish,  
 * distribute, sublicense, and#or sell copies of the Software, and to   
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:                                            
 *                                                                      
 * The above copyright notice and this permission notice shall be       
 * included in all copies or substantial portions of the Software.      
 *                                                                      
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,      
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF   
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR    
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.                                      
 */
//Marks the right margin of code *******************************************************************
package com.rmc.demos.frametagmetadatademo.managers
{
	import com.rmc.demos.frametagmetadatademo.display.preloaders.IPreloader;
	import com.rmc.demos.frametagmetadatademo.display.preloaders.Preloader;
  
  //--------------------------------------
  //  Imports
  //--------------------------------------
  import flash.display.DisplayObject;
  import flash.display.MovieClip;
  import flash.display.StageAlign;
  import flash.display.StageScaleMode;
  import flash.events.Event;
  import flash.utils.getDefinitionByName;
  
  
  //--------------------------------------
  //  Metadata
  //--------------------------------------
  
  
  //--------------------------------------
  //  Class
  //--------------------------------------
  /**
   * <p>The <code>SystemManager</code> is optional. When declared in the main document class's [Frame] tag, 
   * the compiler will call this class first. This class will then show a preloader, and when done, the
   * main document class will be created as a child of this class.</p>
   * 
   * NOTE: This is a NON-DOCUMENTED technique that is assumed stable because its a cornerstone of how 
   * Flex applications work.
   * 
   * NOTE: Its VERY odd how this all works. But it works.
   * 
   * NOTE: Great article that inspired this solution: http://www.bit-101.com/blog/?p=946
   * 
   * <p>AUTHOR  : Samuel Asher Rivello</p>
   * <p>COMPANY : RivelloMultimediaConsulting.com</p>
   * <p>EMAIL   : info@RivelloMultimediaConsulting.com</p>
   * <p>CREATION DATE    : Apr 7, 2011</p>
   * 
   * @example Here is a code example. 
   *  
   * <listing version="3.0">
   * <p>Describe this code example.</p>
   *
   * </listing>
   *
   */
  public class SystemManager extends MovieClip
  {		
    
    //--------------------------------------
    //  Properties
    //--------------------------------------
    //PUBLIC GETTER/SETTERS
    
    //PUBLIC STATIC
    /**
     * Get an instance of the singleton class.
     * 
     * NOTE: This is not a textbook 'Singleton' as Flash automatically creates an instance of it.
     *       The goal of mine here is just to expose a 'getInstance()' so the REST of the application
     *       can respect it *like* a singleton, i.e. not reinstatiating it.
     *  
     * @return <code>SystemManager</code> The instance to return
     * 
     */
    public static function getInstance() : SystemManager  
    {
      return SystemManager._INSTANCE;
    }
    
    
    //PUBLIC STATIC
    /**
     * Determines if this class is initialized. Basically if the class has been used at all.
     * 
     * NOTE: This helps determine within the API if the game did or did not use the
     * SystemManager. Since its optional to use SystemManager a few key places internally
     * in the API want to know.
     * 
     */		
    public static function get isInitialized () 						: Boolean 	{ 	return _isInitialized_boolean; 		}
    private static var _isInitialized_boolean : Boolean = false;
    
    /**
     * A property manually set during initialization here. May be usefull later.
     * 
     */		
    private static var _INSTANCE	: SystemManager;
    
    //PRIVATE
    /**
     * The preloader to be shown while the game loads.
     * 
     */
    private var _iPreloader:IPreloader;   
    
    /**
     * The class to be created dynamically as the main document class.
     * 
     */
    private var _mainDocumentClassName_str:String;
    
    //--------------------------------------
    //  Constructor
    //--------------------------------------
    
    
    /**
     * This is the constructor.
     * 
     */
    public function SystemManager()
    {
      //SUPER
      super(); 
      
      	FrameTagMetadataDemo
      
      _mainDocumentClassName_str = "FrameTagMetadataDemo"; //TODO, HOW DO I DYNAMICALLY PASS THIS IN? OR ACCESS IT FROM HERE?
      
      //METHODS
      _initializeSystemManager ();
      
    }
    
    //--------------------------------------
    //  Methods
    //--------------------------------------		
    //PUBLIC	
    
    //PRIVATE
    /**
     *  Create a class dynamically based on the name of the current swf file.
     * 
     * NOTE: This requires the swf name match the master class (very very likely)
     * 
     *  @return Object of the class
     * 
     */
    private function _getDesiredMainDocumentClass():Object
    {
      //DETERMINE THE DESIRED MAIN CLASS BY ASSUMING it matches the swf name (minus the ".swf")
      var mainClassName_str:String;
      
      if (_mainDocumentClassName_str != null) {
        mainClassName_str = _mainDocumentClassName_str;
      } else {
        var url_str:String      = loaderInfo.loaderURL;
        var dotIndex_int:int    = url_str.lastIndexOf(".");
        var slashIndex_int:int  = url_str.lastIndexOf("/");
        mainClassName_str = url_str.substring(slashIndex_int + 1, dotIndex_int);
      }
      
      //var t : TextField = new TextField ()
      //t.htmlText = mainClassName_str;
      //addChild(t);
      
      //DYNAMICALLY CREATE THE CLASS
      var mainClass:Class = Class(getDefinitionByName(mainClassName_str));
      return mainClass ? new mainClass() : null;
    }
    
    //PRIVATE	
    /**
     * Initialize the SystemManager
     * 
     */
    private function _initializeSystemManager():void
    {
      //BOOLEAN
      SystemManager._INSTANCE = this;
      SystemManager._isInitialized_boolean = true;
      
      //EVENTS
      addEventListener(Event.ENTER_FRAME, onEnterFrame);
      
      //VARIABLES
      
      //PROPERTIES
      stage.scaleMode = StageScaleMode.NO_SCALE;
      stage.align = StageAlign.TOP_LEFT;
      
      //STAGE
      
      //METHODS
      stop();
      _initializePreloader ();
      
    }
    
    /**
     * Initialize the Preloader
     * 
     */
    private function _initializePreloader():void
    {
      //TODO, MAKE THIS CUSTOMIZABLE, BY EITHER COMPILER PARAMETER OR METADATA
      //SINCE I DON'T THINK I CAN PASS PARAMETERS THROUGH [TAG] METADATA
      _iPreloader = new Preloader();
      _iPreloader.show();
      addChild(_iPreloader as DisplayObject);
      
    }
    
    
    /**
     * Initialize the Main Application
     * 
     * @return void
     * 
     */
    private function _initializeMainApplication():void
    {
      
      addChild( _getDesiredMainDocumentClass() as DisplayObject );
      
    }
    
    //--------------------------------------
    //  Event Handlers
    //--------------------------------------		
    /**
     * Handles the event: <code>Event.ENTER_FRAME</code>.
     * 
     * @param event <code>Event</code> The incoming event payload.
     *  
     * @return void
     * 
     */
    public function onEnterFrame(event:Event):void
    {
      
      if(framesLoaded == totalFrames)
      {
        removeEventListener(Event.ENTER_FRAME, onEnterFrame);
        
        //REMOVE PRELOADER
        _iPreloader.hide();
        _iPreloader.destroy();
        
        //JUMP TO FRAME 2 (WHERE WE SIT FOREVER AFTER)
        nextFrame();
        
        //INITIALIZE THE MAIN APP AS A CHILD OF THIS CLASS
        _initializeMainApplication();
        
      } else {
        
        //NEW
        var percent_num:Number = root.loaderInfo.bytesLoaded / root.loaderInfo.bytesTotal;
        _iPreloader.update (percent_num);
      }
    }
    
  }
}


